@@ -5,6 +5,13 @@ from kryptools import Matrix, Zmod
 
 def test_Matrix():
     M = Matrix([[1, 2, 3], [4, 5, 6], [7, 8, 12]])
+    Z = M.zeros()
+    assert M
+    assert M == M
+    assert not Z
+    Z[0] = 1
+    assert Z
+    assert Z != Z.zeros()
     assert len(M) == 3 * 3
     assert M[0] == 1
     assert M[:] == [1, 2, 3, 4, 5, 6, 7, 8, 12]
